/* mnote-data.c
 *
 * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details. 
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "config.h"
#include "mnote-data.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <canon/mnote-canon-data.h>
#include <fuji/mnote-fuji-data.h>
#include <olympus/mnote-olympus-data.h>
#include <pentax/mnote-pentax-data.h>

struct _MNoteDataPrivate {
	ExifByteOrder order;

	unsigned int ref_count;
};

void
mnote_data_construct (MNoteData *note)
{
	if (!note)
		return;

	note->priv = malloc (sizeof (MNoteDataPrivate));
	if (!note->priv)
		return;
	memset (note->priv, 0, sizeof (MNoteDataPrivate));

	note->priv->ref_count = 1;
}

void
mnote_data_ref (MNoteData *note)
{
	if (!note || !note->priv)
		return;

	note->priv->ref_count++;
}

void
mnote_data_unref (MNoteData *note)
{
	if (!note || !note->priv)
		return;

	note->priv->ref_count--;
	if (!note->priv->ref_count)
		mnote_data_free (note);
}

void
mnote_data_free (MNoteData *note)
{
	if (!note)
		return;

	if (note->priv) {
		free (note->priv);
		note->priv = NULL;
	}

	free (note);
}

static void
mnote_data_save_data (MNoteData *note,
		     unsigned char **data, unsigned int *size)
{
	if (!note)
		return;

	if (note->methods.save_data)
		note->methods.save_data (note, data, size);
}

static void
mnote_data_load_data (MNoteData *note,
		     const unsigned char *data, unsigned int size)
{
	if (!note)
		return;

	if (note->methods.load_data)
		note->methods.load_data (note, data, size);
}

MNoteData *
mnote_data_new_from_data (const unsigned char *data, unsigned int size)
{
	MNoteData *note = NULL;

	if (!size || !data)
		return (NULL);

	/* Canon notes begin with 0x0000 */
	if ((size > 1) && (data[0] == 0x00) && (data[1] == 0x00))
		note = mnote_canon_data_new ();

	/* Olympus notes begin with "OLYMP" */
	else if ((size >= 5) && !memcmp (data, "OLYMP", 5)) {
		note = mnote_olympus_data_new ();

	/* Pentax notes have 27 tag entries */
	} else if (size >= 2 && (data[0] == 0x00) && (data[1] == 0x1b)) {
		note = mnote_pentax_data_new ();

	} else {
		note = NULL;
	}

	if (note)
		mnote_data_load_data (note, data, size);

	return (note);
}

void
mnote_data_save_to_data (MNoteData *note,
			unsigned char **data, unsigned int *size)
{
	if (!note || !size || !data)
		return;

	mnote_data_save_data(note, data, size);
}

const char *
mnote_tag_get_name (MNoteData *note, MNoteTag tag)
{
	if (!note || !note->methods.get_tag_name)
		return (NULL);

	return (note->methods.get_tag_name (tag));
}

const char *
mnote_tag_get_title (MNoteData *note, MNoteTag tag)
{
	if (!note || !note->methods.get_tag_title)
		return (NULL);

	return (note->methods.get_tag_title (tag));
}

const char *
mnote_tag_get_description (MNoteData *note, MNoteTag tag)
{
	if (!note || !note->methods.get_tag_description)
		return (NULL);

	return (note->methods.get_tag_description (tag));
}

char *
mnote_data_get_value (MNoteData *note, MNoteTag tag)
{
	if (!note || !note->methods.get_value)
		return (NULL);

	return (note->methods.get_value (note, tag));
}

void
mnote_data_set_byte_order (MNoteData *note, ExifByteOrder order)
{
	if (!note || !note->priv)
		return;
	
	note->priv->order = order;
	
	if (!note->methods.set_order)
		return;

	note->methods.set_order (note, order);
}

ExifByteOrder
mnote_data_get_byte_order (MNoteData *note)
{
	if (!note || !note->priv)
		return (0);

	return (note->priv->order);
}

void
mnote_data_foreach_entry (MNoteData *note, MNoteDataForeachEntryFunc func, void *user_data)
{
	if (!note || !note->methods.foreach_entry)
		return;

	note->methods.foreach_entry (note, func, user_data);
}

void
mnote_data_dump_entry (MNoteData *note, MNoteTag tag, unsigned int indent)
{
	MNoteEntry *entry;

	if (!note || !note->methods.get_entry || !note->methods.dump_entry)
		return;

	entry = note->methods.get_entry(note, tag);

	if (!entry)
		return;

	note->methods.dump_entry(entry, indent);
}

void
mnote_data_dump (MNoteData *note, unsigned int indent)
{
	char buf[1024];
	unsigned int i;

	if (!note || !note->methods.get_count || !note->methods.dump_entry)
		return;

	for (i = 0; i < 2 * indent; i++)
		buf[i] = ' ';
	buf[i] = '\0';

	printf ("%sDumping MakerNote content (%i entries)...\n", buf,
		note->methods.get_count(note));

	mnote_data_foreach_entry (note, (MNoteDataForeachEntryFunc)mnote_data_dump_entry, (void *)indent);
}
