/* mnote-olympus-data.c
 *
 * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details. 
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "config.h"
#include "mnote-olympus-data.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <libexif/exif-byte-order.h>
#include <libexif/exif-utils.h>

#define DEBUG

static void
mnote_olympus_data_free (MNoteData *n)
{
	MNoteOlympusData *note = (MNoteOlympusData *) n;
	unsigned int i;

	if (note->entries) {
		for (i = 0; i < note->count; i++)
			mnote_olympus_entry_unref (note->entries[i]);
		free (note->entries);
		note->entries = NULL;
		note->count = 0;
	}
}

static void
mnote_olympus_data_add_entry (MNoteOlympusData *note, MNoteOlympusEntry *entry)
{
	if (!note || !entry)
		return;

	entry->parent = note;
	note->entries = realloc (note->entries,
				sizeof (MNoteOlympusEntry) * (note->count + 1));
	note->entries[note->count] = entry;
	mnote_olympus_entry_ref (entry);
	note->count++;
}

static char *
mnote_olympus_data_get_value (MNoteData *n, MNoteTag t)
{
	char *value = NULL;
	unsigned int i, count;

	MNoteOlympusData *note = (MNoteOlympusData *) n;
	MNoteOlympusTag tag = (MNoteOlympusTag) t;

	for (count = i = 0; !value && i < note->count; i++) {
		if (note->entries[i]->tag == tag) {
			value = mnote_olympus_entry_get_value (note->entries[i]);
		}
	}

	return (value);
}

static void
mnote_olympus_data_load_data_entry (MNoteOlympusData *note,
				   MNoteOlympusEntry *entry,
				   const unsigned char *d,
				   unsigned int size, unsigned int offset)
{
	unsigned int s, doff;
	ExifByteOrder order = mnote_data_get_byte_order ((MNoteData *) note);

	entry->tag        = exif_get_short (d + offset + 0, order);
	entry->format     = exif_get_short (d + offset + 2, order);
	entry->components = exif_get_long  (d + offset + 4, order);

        /*
         * Size? If bigger than 4 bytes, the actual data is not
         * in the entry but somewhere else (offset).
         */
        s = exif_format_get_size (entry->format) * entry->components;
        if (!s)
                return;
        if (s > 4)
                doff = exif_get_long (d + offset + 8, order);
        else
                doff = offset + 8;

        /* Sanity check */
        if (size < doff + s)
                return;

        entry->data = malloc (sizeof (char) * s);
        if (!entry->data)
                return;
        entry->size = s;
        memcpy (entry->data, d + doff, s);
}

static void
mnote_olympus_data_load_data (MNoteData *en, const unsigned char *data,
			     unsigned int size)
{
	MNoteOlympusData *note = (MNoteOlympusData *) en;
	MNoteOlympusTag tag;
	MNoteOlympusEntry *entry;
	unsigned int i, n;
	ExifByteOrder order = mnote_data_get_byte_order (en);

	/* Verify the header */
	if ((size < 5) || memcmp (data, "OLYMP", 5))
		return;
	data += 5;
	size -= 5;

	/* 2 unknown bytes */
	if ((size < 2) || (data[0] != 0x00) || (data[1] != 0x01))
		return;
	data += 2;
	size -= 2;

	/* Number of entries */
	if (size < 2)
		return;
	n = exif_get_short (data, order);
#ifdef DEBUG
	printf ("Reading %i entries...\n", n);
#endif
	data += 2;
	size -= 2;

	/* 2 unknown bytes */
	if ((size < 2) || (data[0] != 0x00) || (data[1] != 0x00))
		return;
	data += 2;
	size -= 2;

	for (i = 0; i < n; i++) {
		tag = exif_get_short (data + 12 * i, order);
#ifdef DEBUG
		printf ("Loading entry '%s' (%i)...\n",
			mnote_olympus_tag_get_name (tag), i + 1);
#endif
		entry = mnote_olympus_entry_new ();
		mnote_olympus_data_add_entry (note, entry);
		mnote_olympus_data_load_data_entry (note, entry, data, size,
						   12 * i);
		mnote_olympus_entry_unref (entry);
	}
}

MNoteData *
mnote_olympus_data_new (void)
{
	MNoteOlympusData *note;

	note = malloc (sizeof (MNoteOlympusData));
	if (!note)
		return (NULL);
	memset (note, 0, sizeof (MNoteOlympusData));
	mnote_data_construct ((MNoteData *) note);
	((MNoteData *) note)->methods.free = mnote_olympus_data_free;
	((MNoteData *) note)->methods.load_data = mnote_olympus_data_load_data;
	((MNoteData *) note)->methods.get_tag_name = mnote_olympus_tag_get_name;
	((MNoteData *) note)->methods.get_value = mnote_olympus_data_get_value;

	return ((MNoteData *) note);
}
