/* mnote-pentax-data.c
 *
 * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details. 
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "config.h"
#include "mnote-pentax-data.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <libexif/exif-byte-order.h>
#include <libexif/exif-utils.h>

/* #define DEBUG */

static void
mnote_pentax_data_free (MNoteData *n)
{
	MNotePentaxData *note = (MNotePentaxData *) n;
	unsigned int i;

	if (note->entries) {
		for (i = 0; i < note->count; i++)
			mnote_pentax_entry_unref (note->entries[i]);
		free (note->entries);
		note->entries = NULL;
		note->count = 0;
	}
}

static void
mnote_pentax_data_add_entry (MNotePentaxData *note, MNotePentaxEntry *entry)
{
	if (!note || !entry)
		return;

	entry->parent = note;
	note->entries = realloc (note->entries,
				sizeof (MNotePentaxEntry) * (note->count + 1));
	note->entries[note->count] = entry;
	mnote_pentax_entry_ref (entry);
	note->count++;
}

static char *
mnote_pentax_data_get_value (MNoteData *n, MNoteTag t)
{
	char *value = NULL;
	unsigned int i, count;

	MNotePentaxData *note = (MNotePentaxData *) n;
	MNotePentaxTag tag = (MNotePentaxTag) t;

	for (count = i = 0; !value && i < note->count; i++) {
		if (note->entries[i]->tag == tag) {
			value = mnote_pentax_entry_get_value (note->entries[i]);
		}
	}

	return (value);
}

static void
mnote_pentax_data_save_data (MNoteData *n, unsigned char **data,
			     unsigned int *size)
{
	MNotePentaxData *note = (MNotePentaxData *) n;

	note = NULL;
}

static void
mnote_pentax_data_load_data_entry (MNotePentaxData *note,
				  MNotePentaxEntry *entry,
				  const unsigned char *d,
				  unsigned int size, unsigned int offset)
{
	unsigned int s, doff;
	ExifByteOrder order = mnote_data_get_byte_order ((MNoteData *) note);

	entry->tag        = exif_get_short (d + offset + 0, order);
	entry->format     = exif_get_short (d + offset + 2, order);
	entry->components = exif_get_long  (d + offset + 4, order);

        /*
         * Size? If bigger than 4 bytes, the actual data is not
         * in the entry but somewhere else (offset).
         */
        s = exif_format_get_size (entry->format) * entry->components;
        if (!s)
                return;
        if (s > 4)
                doff = exif_get_long (d + offset + 8, order);
        else
                doff = offset + 8;

        /* Sanity check */
        if (size < doff + s)
                return;

        entry->data = malloc (sizeof (char) * s);
        if (!entry->data)
                return;
        entry->size = s;
        memcpy (entry->data, d + doff, s);
}

static void
mnote_pentax_data_load_data (MNoteData *en, const unsigned char *data,
			     unsigned int size)
{
	MNotePentaxData *note = (MNotePentaxData *) en;
	MNotePentaxTag tag;
	MNotePentaxEntry *entry;
	unsigned int i, n;
	ExifByteOrder order = mnote_data_get_byte_order (en);

	/* Number of entries */
	if (size < 2)
		return;
	n = exif_get_short (data, order);
#ifdef DEBUG
	printf ("Reading %i entries...\n", n);
#endif
	data += 2;
	size -= 2;

	for (i = 0; i < n; i++) {
		tag = exif_get_short (data + 12 * i, order);
#ifdef DEBUG
		printf ("Loading entry '%s' (%i)...\n",
			mnote_pentax_tag_get_name (tag), i + 1);
#endif
		entry = mnote_pentax_entry_new ();
		mnote_pentax_data_add_entry (note, entry);
		mnote_pentax_data_load_data_entry (note, entry, data, size,
						   12 * i);
		mnote_pentax_entry_unref (entry);
	}
}

static unsigned int
mnote_pentax_get_count (MNoteData *n)
{
	MNotePentaxData *note = (MNotePentaxData *) n;

	return (note->count);
}

static MNoteEntry *
mnote_pentax_get_entry (MNoteData *n, MNoteTag t)
{
	MNotePentaxData *note = (MNotePentaxData *) n;
	MNotePentaxTag tag = (MNotePentaxTag) t;
	unsigned int i;
	MNoteEntry *entry = NULL;

	for (i = 0; !entry && i < note->count; i++) {
		if (note->entries[i]->tag == tag) {
			entry = note->entries[i];
		}
	}

	return (entry);
}

static void
mnote_pentax_foreach_entry (MNoteData *n, MNoteDataForeachEntryFunc func, void *data)
{
	MNotePentaxData *note = (MNotePentaxData *) n;
	unsigned int i;

	if (!note || !func)
		return;

	for (i = 0; i < note->count; i++)
		func (n, (MNoteTag)note->entries[i]->tag, data);
}

MNoteData *
mnote_pentax_data_new (void)
{
	MNotePentaxData *note;

	note = malloc (sizeof (MNotePentaxData));
	if (!note)
		return (NULL);
	memset (note, 0, sizeof (MNotePentaxData));
	mnote_data_construct ((MNoteData *) note);
	((MNoteData *) note)->methods.free = mnote_pentax_data_free;
	((MNoteData *) note)->methods.load_data = mnote_pentax_data_load_data;
	((MNoteData *) note)->methods.save_data = mnote_pentax_data_save_data;
	((MNoteData *) note)->methods.get_count = mnote_pentax_get_count;
	((MNoteData *) note)->methods.get_entry = mnote_pentax_get_entry;
	((MNoteData *) note)->methods.foreach_entry = mnote_pentax_foreach_entry;
	((MNoteData *) note)->methods.get_tag_name = mnote_pentax_tag_get_name;
	((MNoteData *) note)->methods.get_tag_title = mnote_pentax_tag_get_title;
	((MNoteData *) note)->methods.get_tag_description = mnote_pentax_tag_get_description;
	((MNoteData *) note)->methods.get_value = mnote_pentax_data_get_value;
	((MNoteData *) note)->methods.dump_entry = mnote_pentax_entry_dump;

	return ((MNoteData *) note);
}
